package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.*;
import cn.wise.sc.cement.business.mapper.*;
import cn.wise.sc.cement.business.model.*;
import cn.wise.sc.cement.business.model.query.*;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.*;
import cn.wise.sc.cement.business.util.CheckCountUtil;
import cn.wise.sc.cement.business.util.ExcelUtil;
import cn.wise.sc.cement.business.util.RedisUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2021-01-12
 */
@Service
public class DataStatisticsServiceImpl implements IDataStatisticsService {


    @Resource
    private DataStatisticsMapper dataStatisticsMapper;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private IEquipmentService equipmentService;
    @Autowired
    private ISampleService sampleService;
    @Autowired
    private IEntrustService entrustService;
    @Autowired
    private CommonServiceImpl commonService;


    /**
     * 统计概览-重要数值统计
     * @return
     */
    @Override
    public BaseResponse<Map<String, String>> countOverview() {
        Map<String, String> map = new HashMap<>();
        //当前进行的项目
        QueryWrapper<Project> projectWrapper = new QueryWrapper<>();
        projectWrapper.eq("status", 1);
        Integer projectCounts = projectService.count(projectWrapper);
        map.put("projectCounts", projectCounts.toString());

        //实验设备总量
        QueryWrapper<Equipment> equipmentWrapper = new QueryWrapper<>();
        Integer equipmentCounts = equipmentService.count(equipmentWrapper);
        map.put("equipmentCounts", equipmentCounts.toString());

        //样品库存总量
        QueryWrapper<Sample> sampleWrapper = new QueryWrapper<>();
        sampleWrapper.ne("status", 2);
        Integer sampleCounts = sampleService.count(sampleWrapper);
        map.put("sampleCounts", sampleCounts.toString());

        //委托单数量
        QueryWrapper<Entrust> entrustWrapper = new QueryWrapper<>();
        Integer entrustCounts = entrustService.count(entrustWrapper);
        map.put("entrustCounts", entrustCounts.toString());

        return BaseResponse.okData(map);
    }

    /**
     * 检测项数量统计-按时间
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    public BaseResponse<List<DataStatisticsVo>> countTeamByTime(String startDate, String endDate){
        Map<String, String> dateMap = new HashMap<>();

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");

        if(startDate != null || endDate != null){
            List<String> dateList = commonService.getMonthBetween(startDate, endDate);
            if(dateList != null && dateList.size()>0){
                for(int i=0; i<dateList.size(); i++ ){
                    dateMap.put(dateList.get(i),"0");
                }
            }

        }else{
            Date date = new Date();//获取当前时间
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);

            for(int i=0; i<12; i++){
                date = calendar.getTime();
                String dateTime =  format.format(date);
                dateMap.put(dateTime,"0");
                calendar.add(Calendar.MONTH, -1);
            }
        }

        Map<String, Object> params = new HashMap<>();
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        List<DataStatisticsVo> list = dataStatisticsMapper.countTeamByTime(params);
        for(DataStatisticsVo vo:list){
            if(dateMap.containsKey(vo.getName())){
                dateMap.put(vo.getName(), vo.getValue());
            }
        }
        List<DataStatisticsVo> listResult = new ArrayList<>();
        for(String key:dateMap.keySet()){
            DataStatisticsVo vo = new DataStatisticsVo();
            vo.setName(key);
            vo.setValue(dateMap.get(key));
            listResult.add(vo);
        }

        return BaseResponse.okData(listResult);
    }

    /**
     * 检测项数量统计-按地区
     * @param startDate
     * @param endDate
     * @return
     */
    public BaseResponse<List<DataStatisticsVo>> countTeamByOrigin(String startDate, String endDate){
        Map<String, Object> params = new HashMap<>();
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        List<DataStatisticsVo> list = dataStatisticsMapper.countTeamByOrigin(params);
        return BaseResponse.okData(list);
    }







}
